from __future__ import print_function
import numpy as np
from scipy.signal import fftconvolve


def shift_data(arr):
    min_arr = np.min(arr)
    if min_arr < 0:
        arr -= min_arr
    return arr


def normxcorr2(template, image):
    """
    Computes the normalised cross-correlation of matrices template and image.
    The matrix image must be larger than the matrix template, otherwise they will be swapped
    The resulting matrix contains correlation coefficients and its values may range from -1.0 to 1.0.

    :param template: the smaller matrix to be cross-correlated with the image.
    :param image: the bigger matrix
    :return: N-D matrix contains correlation coefficients and its values may range from -1.0 to 1.0.
    """
    if np.ndim(template) > np.ndim(image) or \
            len([i for i in range(np.ndim(template)) if template.shape[i] > image.shape[i]]) > 0:
        print("Template larger than image. Arguments are swapped.")
        temp = image
        image = template
        template = temp

    template = shift_data(template)
    image = shift_data(image)

    cross_corr = fftconvolve(np.rot90(template, 2), image)
    m, n = template.shape
    mn = m * n

    local_sum_A2 = local_sum(image ** 2, m, n)
    local_sum_A = local_sum(image, m, n)

    diff_local_sums = (local_sum_A2 - (local_sum_A ** 2) / mn)
    denom_A = np.sqrt(np.maximum(diff_local_sums, 0))

    denom_T = np.sqrt(mn - 1) * np.std(template, ddof=1)
    denom = denom_T * denom_A
    numerator = cross_corr - local_sum_A * template.sum() / mn

    out = np.zeros(numerator.shape)
    tol = np.sqrt(np.spacing(np.max(np.abs(denom))))
    i_nonzero = np.where(denom > tol)
    out[i_nonzero] = numerator[i_nonzero] / denom[i_nonzero]

    out[np.where((np.abs(out) - 1) > np.sqrt(np.spacing(1)))] = 0
    return out


def local_sum(A, m, n):
    B = np.pad(A, ((m, m), (n, n)), mode='constant', constant_values=0)
    s = np.cumsum(B, axis=0)
    c = s[m:-1, :] - s[:-m - 1, :]
    s = np.cumsum(c, axis=1)
    local_sum_A = s[:, n:-1] - s[:, :-n - 1]
    return local_sum_A



from __future__ import print_function
from contextlib import contextmanager

import time


@contextmanager
def tictoc(function_name):
    start = time.time()
    yield
    end = time.time()
    print('{0:s}: finished in {1: 9.9f} seconds'.format(function_name, end - start))



import unittest

from scipy.io import loadmat
import numpy as np

from pymlfunc import normxcorr2
from pymlfunc import tictoc


class Test(unittest.TestCase):
    def _testxcorr2(self, template, image, result):
        with tictoc('Template size = {}, image size = {}'.format(template.shape, image.shape)):
            result_ = normxcorr2(template, image)

        self.assertTrue(np.allclose(result_, result))

    def testxcorr2_small(self):
        saved = loadmat('tests/normxcorr2.mat')
        a1 = saved['a1'].astype(np.float)
        c1 = saved['c1'].astype(np.float)
        template1 = saved['template1'].astype(np.float)

        self._testxcorr2(template1, a1, c1)

    def testxcorr2_big(self):
        saved = loadmat('tests/normxcorr2.mat')
        a2 = saved['a2'].astype(np.float)
        c2 = saved['c2'].astype(np.float)
        template2 = saved['template2'].astype(np.float)

        self._testxcorr2(template2, a2, c2)



import setuptools
from codecs import open


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pymlfunc",
    version="0.0.1",
    author="Yukio Fukuzawa",
    author_email="y.fukuzawa@massey.ac.nz",
    description="Implementation of some useful Matlab functions that are missing from numpy/scipy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fzyukio/python-matlab-functions",
    packages=setuptools.find_packages(),
    install_requires=['numpy', 'scipy'],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)